from fastapi import APIRouter, HTTPException, Depends
from sqlalchemy.orm import Session
from datetime import datetime, timedelta
from typing import List

from app.models import Dewar as DewarModel, Slot as SlotModel, LogisticsEvent as LogisticsEventModel
from app.schemas import LogisticsEventCreate, SlotCreate, Slot as SlotSchema
from app.database import get_db

router = APIRouter()

@router.post("/scan-dewar")
async def scan_dewar(event_data: LogisticsEventCreate, db: Session = Depends(get_db)):
    dewar_qr_code = event_data.dewar_qr_code
    location_qr_code = event_data.location_qr_code
    transaction_type = event_data.transaction_type

    dewar = db.query(DewarModel).filter(DewarModel.unique_id == dewar_qr_code).first()
    if not dewar:
        raise HTTPException(status_code=404, detail="Dewar not found")

    if transaction_type == 'incoming':
        slot = db.query(SlotModel).filter(SlotModel.id == location_qr_code).first()
        if not slot or slot.occupied:
            raise HTTPException(status_code=404, detail="Slot not found or already occupied")
        slot.occupied = True
        log_event(db, dewar.id, slot.id, 'incoming')

    elif transaction_type == 'beamline':
        log_event(db, dewar.id, None, 'beamline')

    elif transaction_type == 'outgoing':
        slot = db.query(SlotModel).filter(SlotModel.id == location_qr_code).first()
        if not slot or not slot.occupied:
            raise HTTPException(status_code=404, detail="Slot not found or not occupied")
        slot.occupied = False
        log_event(db, dewar.id, slot.id, 'outgoing')

    elif transaction_type == 'release':
        slot = db.query(SlotModel).filter(SlotModel.id == location_qr_code).first()
        if not slot or not slot.occupied:
            raise HTTPException(status_code=404, detail="Slot not found or not occupied")
        slot.occupied = False
        log_event(db, dewar.id, slot.id, 'released')

    db.commit()
    return {"message": "Status updated successfully"}

def log_event(db: Session, dewar_id: int, slot_id: int, event_type: str):
    new_event = LogisticsEventModel(dewar_id=dewar_id, slot_id=slot_id, event_type=event_type)
    db.add(new_event)

@router.get("/refill-status", response_model=List[SlotSchema])
async def refill_status(db: Session = Depends(get_db)):
    slots_needing_refill = db.query(SlotModel).filter(SlotModel.needs_refill == True).all()
    result = []
    current_time = datetime.utcnow()

    for slot in slots_needing_refill:
        time_until_next_refill = slot.last_refill + timedelta(hours=24) - current_time
        result.append({
            'slot_id': slot.id,
            'needs_refill': slot.needs_refill,
            'time_until_refill': str(time_until_next_refill)
        })

    return result